<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class SituacaoFuncionalRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager       $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function retornaUltimaSituacaoFuncional(
        int     $nrOrg,
        int     $nrOrgPadrao,
        int     $nrVinculom,
        string  $dtCompetenciaSituacaoFuncional
    ): Array {

        $sql = <<<SQL
            SELECT *
            FROM GPE_ALTESITUFUNC A
            JOIN GPE_SITUFUNCM  SFM ON (SFM.NRORG = A.NRORG OR SFM.NRORG = :P_NRORG_PADRAO) AND SFM.NRSITUFUNCM = A.NRSITUFUNCM
            JOIN GPE_SITUFUNCH  SFH ON (SFH.NRORG = A.NRORG OR SFM.NRORG = :P_NRORG_PADRAO) AND SFH.NRSITUFUNCM = A.NRSITUFUNCM AND SFH.DTEFETIVA = (
                SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH SFHH WHERE SFHH.NRORG = SFH.NRORG AND SFHH.NRSITUFUNCM = SFH.NRSITUFUNCM AND SFH.DTEFETIVA <=TRUNC(TO_DATE(:P_DTINISITUFUNC, 'DD/MM/YYYY'), 'MONTH')
            )
            WHERE A.NRORG         = :P_NRORG
              AND A.NRVINCULOM    = :P_NRVINCULOM
              AND A.DTINISITUFUNC = (
                SELECT MAX(B.DTINISITUFUNC)
               FROM GPE_ALTESITUFUNC B
              WHERE B.NRORG       = A.NRORG
                AND B.NRVINCULOM  = A.NRVINCULOM
                AND B.DTINISITUFUNC <= TO_DATE(:P_DTINISITUFUNC, 'DD/MM/YYYY')
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'           => $nrOrg,
                'P_NRVINCULOM'      => $nrVinculom,
                'P_DTINISITUFUNC'   => $dtCompetenciaSituacaoFuncional,
                'P_NRORG_PADRAO'    => $nrOrgPadrao,
            )
        );

        $alteSituFunc = $std->fetch();

        return is_array($alteSituFunc) ? $alteSituFunc : [];
    }

    public function retornaPeriodosEstabilidade(
        int     $nrOrg,
        int     $nrOrgPadrao,
        int     $nrVinculom
    ): Array {

        $sql = <<<SQL
            SELECT *
            FROM GPE_ALTESTABVINC EV
            LEFT JOIN GPE_MOTIVOESTAB ME ON (ME.NRORG = EV.NRORG OR ME.NRORG = :P_NRORG_PADRAO) AND ME.NRMOTIVOESTAB = EV.NRMOTIVOESTAB
            WHERE EV.NRORG = :P_NRORG
                AND EV.NRVINCULOM = :P_NRVINCULOM
            ORDER BY EV.DTINIESTABVINC DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'           => $nrOrg,
                'P_NRVINCULOM'      => $nrVinculom,
                'P_NRORG_PADRAO'    => $nrOrgPadrao,
            )
        );

        $alteAlteEstab = $std->fetchAll();

        return $alteAlteEstab;
    }

    public function setAlteEscala(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO GPE_ALTEESCALA (
                NRORG,              NRALTEESCALA,           NRVINCULOM,
                DTINIESCALA,        DTFIMESCALA,            NRESCALATRABM,
                NRHORDIAM,          NRMOTIVOESCALA,         DSOBSERVACAO,
                DTINCLUSAO,         DTULTATU,               NRORGINCLUSAO,
                CDOPERINCLUSAO,     NRORGULTATU,            CDOPERULTATU,
                NRTURNO,            QTHRLIMITESEPHEXTRA
            )
            VALUES (
                :NRORG,             :NRALTEESCALA,          :NRVINCULOM,
                :DTINIESCALA,       :DTFIMESCALA,           :NRESCALATRABM,
                :NRHORDIAM,         :NRMOTIVOESCALA,        :DSOBSERVACAO,
                :DTINCLUSAO,        :DTULTATU,              :NRORGINCLUSAO,
                :CDOPERINCLUSAO,    :NRORGULTATU,           :CDOPERULTATU,
                :NRTURNO,           :QTHRLIMITESEPHEXTRA
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function setAlteSituacaoFuncional(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO GPE_ALTESITUFUNC (
                NRALTESITUFUNC,     NRORG,              NRMOTIVOAFASTA,
                NRPROXSITUFUNC,     CDTABECDI,          CDDIAGNOST,
                NRSITUFUNCM,        NRVINCULOM,         DTINISITUFUNC,
                DTFIMSITUFUNC,      DTVERIFSITUFUNC,    DTPREVIRETOSITU,
                DTPRORRSITUFUNC,    DTINCLUSAO,         DTULTATU,
                NRORGINCLUSAO,      CDOPERINCLUSAO,     NRORGULTATU,
                CDOPERULTATU,       DSOBSALTSITFUNC,    NRQTDIASITUFUNC,
                DTESTABSITUFUNC,    NRPESSOA,           QTDIASATESTADOEMPR
            )
            VALUES (
                :NRALTESITUFUNC,    :NRORG,             :NRMOTIVOAFASTA,
                :NRPROXSITUFUNC,    :CDTABECDI,         :CDDIAGNOST,
                :NRSITUFUNCM,       :NRVINCULOM,        :DTINISITUFUNC,
                :DTFIMSITUFUNC,     :DTVERIFSITUFUNC,   :DTPREVIRETOSITU,
                :DTPRORRSITUFUNC,   :DTINCLUSAO,        :DTULTATU,
                :NRORGINCLUSAO,     :CDOPERINCLUSAO,    :NRORGULTATU,
                :CDOPERULTATU,      :DSOBSALTSITFUNC,   :NRQTDIASITUFUNC,
                :DTESTABSITUFUNC,   :NRPESSOA,          :QTDIASATESTADOEMPR
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * 
     * 
     * 
     */
    public function getMotivoSituFunc(
        int     $nrOrg,
        int     $nrMotivoOcorFreq   = null,
        int     $nrSituFuncm        = null
    ): Array {

        $sql = <<<SQL
            SELECT *
            FROM PTO_MOTIVOSITUFUNC MS
            WHERE MS.NRORG = :P_NRORG
                AND (
                    (:P_NRMOTIVOOCORFREQ    IS NOT NULL AND MS.NRMOTIVOOCORFREQ = :P_NRMOTIVOOCORFREQ) OR
                    (:P_NRSITUFUNCM         IS NOT NULL AND MS.NRSITUFUNCM      = :P_NRSITUFUNCM)
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'               => $nrOrg,
                'P_NRMOTIVOOCORFREQ'    => $nrMotivoOcorFreq,
                'P_NRSITUFUNCM'         => $nrSituFuncm
            )
        );

        $motivoSituFunc = $std->fetchAll();

        return $motivoSituFunc;
    }
}