<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;
use Util\NovoCodigo;

use Ems\Repository\OcupacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil };

class OcupacaoService {
    /** @var Environment */
    protected $environment;

    /** @var OcupacaoRepository */
    protected $ocupacaoRepository;

    public function __construct(
        OcupacaoRepository $ocupacaoRepository,
        Environment $environment
    ) {
        $this->ocupacaoRepository         = $ocupacaoRepository;
        $this->environment                  = $environment;
    }



    public function getMotivoOcupa() {
        $nrorg = $this->environment->getNrorg();
        return $this->ocupacaoRepository->getMotivoOcupa($nrorg);
    }

    public function getOcupacao() {
        $nrorg = $this->environment->getNrorg();
        return $this->ocupacaoRepository->getOcupacao($nrorg);
    }

    public function setAlteOcupacao(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRALTEOCUPACAO'] =
            $row['NRALTEOCUPACAO'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTEOCUPACAO', 1, 12, true, false);

        $params['NRORG']            = $row['NRORG'] ?? $nrOrg;
        $params['NRVINCULOM']       = (int) $row['NRVINCULOM'];
        $params['DTINIOCUPACAO']    = (string) $row['DTINIOCUPACAO'];
        $params['DTFIMOCUPACAO']    = $row['DTFIMOCUPACAO'] ?? null;
        $params['NROCUPACAOM']      = (int) $row['NROCUPACAOM'];
        $params['NRNIVELOCUPA']     = $row['NRNIVELOCUPA'] ?? null;
        $params['NRMOTIVOOCUPA']    = $row['NRMOTIVOOCUPA'] ?? null;
        $params['NRTPMODALIDSAL']   = $row['NRTPMODALIDSAL'] ?? null;
        $params['DSOBSERVACAO']     = $row['DSOBSERVACAO'] ?? null;
        $params['DTINCLUSAO']       = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']         = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']    = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']   = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']      = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']     = $row['CDOPERULTATU'] ?? null;
        $params['NRTIPOOCUPACAO']   = $row['NRTIPOOCUPACAO'] ?? null;
        $params['NRFAIXA']          = $row['NRFAIXA'] ?? null;

        $this->ocupacaoRepository->setAlteOcupacao($params);

        return $params['NRALTEOCUPACAO'];
    }
}
