<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\SendEmail\SendEmail;
use Util\Environment;

use Ems\Repository\PessoaRepository;
use Ems\Util\{ Utilities, ArrayUtil, ExpectedException};
use Util\NovoCodigo;

class PessoaService {
    /** @var PessoaRepository */
    protected $pessoaRepository;

    /** @var Environment */
    protected $environment;

    public function __construct(
        PessoaRepository    $pessoaRepository,
        Environment         $environment
    ) {
        $this->pessoaRepository = $pessoaRepository;
        $this->environment      = $environment;
    }



    public function getPessoa($nrvinculom, $dtmescompetenc) {
        $nrorg = $this->environment->getNrorg();

        return $this->pessoaRepository->getPessoa($nrorg, $nrvinculom, $dtmescompetenc);
    }

    public function getPessoaByCpf(
        string $cpf
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $pessoa = $this->pessoaRepository->getPessoaByCpf($nrOrg, $cpf);

        return $pessoa;
    }

    public function getPessoaByPis(
        string $pis
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $pessoa = $this->pessoaRepository->getPessoaByPis($nrOrg, $pis);

        return $pessoa;
    }

    public function get(
        int $nrPessoa,
        $throwException = true
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $pessoa = $this->pessoaRepository->get($nrOrg, $nrPessoa);

        if (!$pessoa && $throwException) {
            throw new ExpectedException('Não foi encontrada a pessoa de número ' . $nrPessoa);
        }

        return $pessoa;
    }

    public function setPessoa(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRPESSOA'] =
            $row['NRPESSOA'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_PESSOA', 1, 12, true, false);

        $params['NRORG']            = $row['NRORG'] ?? $nrOrg;
        $params['NRPARCNEGOCIO']    = (int) $row['NRPARCNEGOCIO'];
        $params['DTINCLUSAO']       = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']         = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']    = $row['NRORG'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']   = $row['CDOPERINCLUSAO'] ?? $cdOperador ?? '1';
        $params['NRORGULTATU']      = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']     = $row['CDOPERULTATU'] ?? null;
        $params['DTESOCIAL']        = $row['DTESOCIAL'] ?? null;

        $this->pessoaRepository->setPessoa($params);

        return $params['NRPESSOA'];
    }

    public function setPessoaH(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        if (!isset($params['NRPESSOAH'])) {
            $params['NRPESSOAH'] = NovoCodigo::geraCodigo($nrOrg, 'GPE_PESSOAH', 1, 12, true, false);
        }

        $params['NRPESSOA'] =           $row['NRPESSOA'] ?? null;
        $params['NRORG'] =              $nrOrg;
        $params['DTMESCOMPETENC'] =     $row['DTMESCOMPETENC'] ?? null;
        $params['NMPESSOA'] =           $row['NMPESSOA'] ?? null;
        $params['DSPRONOMETRAT'] =      $row['DSPRONOMETRAT'] ?? null;
        $params['DSAPELIDOPESS'] =      $row['DSAPELIDOPESS'] ?? null;
        $params['DSPRINOMEPESS'] =      $row['DSPRINOMEPESS'] ?? null;
        $params['DSULTNOMEPESS'] =      $row['DSULTNOMEPESS'] ?? null;
        $params['DSNOMEMEIOPESS'] =     $row['DSNOMEMEIOPESS'] ?? null;
        $params['CDESTACIVIL'] =        $row['CDESTACIVIL'] ?? null;
        $params['NRCONDFISPES'] =       $row['NRCONDFISPES'] ?? null;
        $params['NRANOCHEGAPAIS'] =     $row['NRANOCHEGAPAIS'] ?? null;
        $params['NRDOCUESTRANG'] =      $row['NRDOCUESTRANG'] ?? null;
        $params['IDPOSVISTOPERM'] =     $row['IDPOSVISTOPERM'] ?? null;
        $params['DTPERMAVISTO'] =       $row['DTPERMAVISTO'] ?? null;
        $params['IDPESNATURALIZA'] =    $row['IDPESNATURALIZA'] ?? null;
        $params['DTNATURALIZAPES'] =    $row['DTNATURALIZAPES'] ?? null;
        $params['NRCPFPESSOA'] =        $row['NRCPFPESSOA'] ?? null;
        $params['NRREGCTPSPES'] =       $row['NRREGCTPSPES'] ?? null;
        $params['NRCTPSPESSOA'] =       $row['NRCTPSPESSOA'] ?? null;
        $params['NRSERIECTPSPES'] =     $row['NRSERIECTPSPES'] ?? null;
        $params['SGUFCTPSPES'] =        $row['SGUFCTPSPES'] ?? null;
        $params['DTCTPSPESSOA'] =       $row['DTCTPSPESSOA'] ?? null;
        $params['NRPISPASEPPES'] =      $row['NRPISPASEPPES'] ?? null;
        $params['DTINSCPISPASEP'] =     $row['DTINSCPISPASEP'] ?? null;
        $params['NRINSCINSSPES'] =      $row['NRINSCINSSPES'] ?? null;
        $params['NRINSCISSPES'] =       $row['NRINSCISSPES'] ?? null;
        $params['NRCERTNASCPES'] =      $row['NRCERTNASCPES'] ?? null;
        $params['NRCERTCASAPES'] =      $row['NRCERTCASAPES'] ?? null;
        $params['DTCASAPESSOA'] =       $row['DTCASAPESSOA'] ?? null;
        $params['IDISENTITUELEI'] =     $row['IDISENTITUELEI'] ?? null;
        $params['NRTITUELEIPES'] =      $row['NRTITUELEIPES'] ?? null;
        $params['NRSECAELEIPES'] =      $row['NRSECAELEIPES'] ?? null;
        $params['NRZONAELEIPES'] =      $row['NRZONAELEIPES'] ?? null;
        $params['NRCARTHABPES'] =       $row['NRCARTHABPES'] ?? null;
        $params['DTHABCARTPES'] =       $row['DTHABCARTPES'] ?? null;
        $params['DTVALHABCARTPES'] =    $row['DTVALHABCARTPES'] ?? null;
        $params['DSCATEGHABCART'] =     $row['DSCATEGHABCART'] ?? null;
        $params['NRCATMILITARPE'] =     $row['NRCATMILITARPE'] ?? null;
        $params['NRCERTRESEPES'] =      $row['NRCERTRESEPES'] ?? null;
        $params['DSCATEGCERTRESE'] =    $row['DSCATEGCERTRESE'] ?? null;
        $params['DTCERTRESEPES'] =      $row['DTCERTRESEPES'] ?? null;
        $params['CDEXCERTRESEPES'] =    $row['CDEXCERTRESEPES'] ?? null;
        $params['NRCONSELHOREG'] =      $row['NRCONSELHOREG'] ?? null;
        $params['NRINSCRCONSREG'] =     $row['NRINSCRCONSREG'] ?? null;
        $params['DTEXCONSREG'] =        $row['DTEXCONSREG'] ?? null;
        $params['SGUFCONSREG'] =        $row['SGUFCONSREG'] ?? null;
        $params['DTINCLUSAO'] =         $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU'] =           $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO'] =      $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO'] =     $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU'] =        $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU'] =       $row['CDOPERULTATU'] ?? null;
        $params['NRRACAPESSOA'] =       $row['NRRACAPESSOA'] ?? null;
        $params['IDGRUPSANGUPES'] =     $row['IDGRUPSANGUPES'] ?? null;
        $params['IDFATORRHPES'] =       $row['IDFATORRHPES'] ?? null;
        $params['CDPAIS'] =             $row['CDPAIS'] ?? null;
        $params['SGESTADO'] =           $row['SGESTADO'] ?? null;
        $params['CDMUNICIPIO'] =        $row['CDMUNICIPIO'] ?? null;
        $params['DTNASCPESSOA'] =       $row['DTNASCPESSOA'] ?? null;
        $params['IDSEXOPESSOA'] =       $row['IDSEXOPESSOA'] ?? null;
        $params['CDTPBAIXAPAG'] =       $row['CDTPBAIXAPAG'] ?? null;
        $params['NRRGPESSOA'] =         $row['NRRGPESSOA'] ?? null;
        $params['CDEXRGPESSOA'] =       $row['CDEXRGPESSOA'] ?? null;
        $params['SGUFRGPESSOA'] =       $row['SGUFRGPESSOA'] ?? null;
        $params['DSLOCALEXRGPES'] =     $row['DSLOCALEXRGPES'] ?? null;
        $params['DTEXRGPESSOA'] =       $row['DTEXRGPESSOA'] ?? null;
        $params['NRNACIONALID'] =       $row['NRNACIONALID'] ?? 1;
        $params['NRGRAUINSTR'] =        $row['NRGRAUINSTR'] ?? null;
        $params['DSCOMPLCERTNASC'] =    $row['DSCOMPLCERTNASC'] ?? null;
        $params['NRLIVROCERTNASC'] =    $row['NRLIVROCERTNASC'] ?? null;
        $params['NRFOLHACERTNASC'] =    $row['NRFOLHACERTNASC'] ?? null;
        $params['DSCAMINHOFOTO'] =      $row['DSCAMINHOFOTO'] ?? null;
        $params['IDPOSSUIFILHO'] =      $row['IDPOSSUIFILHO'] ?? null;
        $params['NRIDADEFILHO'] =       $row['NRIDADEFILHO'] ?? null;
        $params['DTINIVIGENCIA'] =      $row['DTINIVIGENCIA'] ?? null;
        $params['DTFIMVIGENCIA'] =      $row['DTFIMVIGENCIA'] ?? null;
        $params['NRLIVROCERTCASA'] =    $row['NRLIVROCERTCASA'] ?? null;
        $params['NRFOLHACERTCASA'] =    $row['NRFOLHACERTCASA'] ?? null;
        $params['SGUFHABCARPES'] =      $row['SGUFHABCARPES'] ?? null;
        $params['DSEXPDOCESTR'] =       $row['DSEXPDOCESTR'] ?? null;
        $params['NRCLASSESTRANG'] =     $row['NRCLASSESTRANG'] ?? null;
        $params['NRCERTOBITO'] =        $row['NRCERTOBITO'] ?? null;
        $params['DTCHEGADAPAIS'] =      $row['DTCHEGADAPAIS'] ?? null;
        $params['NRSUSPESSOA'] =        $row['NRSUSPESSOA'] ?? null;
        $params['NRNITPESSOA'] =        $row['NRNITPESSOA'] ?? null;
        $params['DTADMISSAOPRE'] =      $row['DTADMISSAOPRE'] ?? null;
        $params['CDRECIBOADMPRE'] =     $row['CDRECIBOADMPRE'] ?? null;
        $params['IDCOTAPCD'] =          $row['IDCOTAPCD'] ?? null;

        $this->pessoaRepository->setPessoaH($params);

        return $params['NRPESSOAH'];
    }

    /**
     * Separa o nome passado em primeiro, meio e último nome.
     * Ex: Reynaldo Silva Guedes Pinto => ["Reynaldo", "Silva Guedes", "Pinto"]
     *
     * @param string $nome O nome da pessoa.
     *
     * @return string[] O nome da pessoa dividido em um array, onde os indexes são:
     * primeiro nome, nome do meio e último nome, respectivamente.
     */
    public static function separarNomePessoa(
        string $nome
    ): array {
        $primeiroNome = null;
        $nomeMeio = null;
        $ultimoNome = null;

        $nomesMeio = [];

        // Remove os espaços em branco do início e do fim e da string
        // e cria um array com os nomes
        $nomes = explode(' ', trim($nome));

        if (!$nomes) {
            return false;
        }

        $lastIndex = count($nomes) - 1;

        foreach ($nomes as $index => $nomeFromArray) {
            if ($index === 0) {
                $primeiroNome = $nomeFromArray;
            } elseif ($index === $lastIndex) {
                $ultimoNome = $nomeFromArray;
            } else {
                array_push($nomesMeio, $nomeFromArray);
            }
        }

        if (count($nomesMeio) > 0) {
            $nomeMeio = '';
            $nomesMeioLastIndex = count($nomesMeio) - 1;

            // Junta todos os nomes do meio
            foreach ($nomesMeio as $index => $item) {
                $nomeMeio .= $item;

                // Adiciona espaço após o nome caso não seja o último
                if ($index !== $nomesMeioLastIndex) {
                    $nomeMeio .= ' ';
                }
            }
        }

        $nomeSeparado = [$primeiroNome, $nomeMeio, $ultimoNome];

        return $nomeSeparado;
    }

    public function getCdEstadoCivilByNm(
        string $nmEstadoCivil
    ) {
        $estadoCivil = $this->pessoaRepository->getEstadoCivilByNm(
            $nmEstadoCivil
        );

        $cdEstadoCivil = $estadoCivil['CDESTACIVIL'] ?? null;
        return $cdEstadoCivil;
    }

    public function getNrRacaByNm(
        string $nmRaca
    ) {
        $raca = $this->pessoaRepository->getRacaByNm(
            $nmRaca
        );

        $nrRaca = $raca['NRRACAPESSOA'] ?? null;
        return $nrRaca;
    }
}
