<?php

namespace Util;

/**
 * Controlador de Alertas. Permite o armazenamento de mensagens geradas ao longo da execucao de processos/rotinas expecificas.
 * 
 */

class ControladorAlerta {
    /** @var string */
    private $id;
    /** @var array */
    private $mensagens;
    
    /** @var ControladorAlerta */
    private static $instance = null;
    
    /** @var string */
    const TIPO_ERRO = 'ERRO';
    /** @var string */
    const TIPO_ALERTA = 'ALERTA';
    /** @var string */
    const TIPO_SUCCESS = 'SUCESSO';
    
    public function __construct() {
        $this->mensagens = [];
        $this->id = md5(time());
    }
    
    /**
     * Obtem as mensagens armazenadas
     * @return array
     */
    public function getMensagens(): array {
        return $this->mensagens;
    }
    
    /**
     * Obtem o id do processo
     * @return string
     */
    public function getId(): string {
        return $this->id;
    }
    
    /**
     * Adiciona uma nova mensagem ao processo
     * @param string $texto
     * @param string $tipo
     * @return void
     */
    public function adicionaMensagem(string $texto, string $tipo = self::TIPO_ALERTA): void {
        $this->mensagens[] = [
            'TEXTO' => $texto,
            'TIPO' => $tipo,
        ];
    }
    
    /**
     * Adiciona uma nova mensagem ao processo da instancia atual
     * @param string $texto
     * @param string $tipo
     * @return void
     */
    public static function adicionaMensagemProcessoAtual(string $texto, string $tipo = self::TIPO_ALERTA) {
        self::verificaInstanciaValida();
        self::$instance->adicionaMensagem($texto, $tipo);
    }
    
    /**
     * Obtem as mensagens do processo da instancia atual
     * @return array
     */
    public static function getMensagensProcessoAtual(): array {
        self::verificaInstanciaValida();
        return self::$instance->getMensagens();
    }
    
    /**
     * Instancia o processo para a gravacao das mensagens
     * @return void
     */
    public static function iniciaProcesso(): void {
        self::$instance = new self();
    }
    
    /**
     * Obtem a instancia atual do processo
     * @return ControladorAlerta
     */
    public static function getInstance(): ControladorAlerta {
        return self::$instance;
    }
    
    /**
     * Verifica se existe uma instancia valida do proceesso atual. Caso nao exista uma nova e iniciada
     * @return void
     */
    private static function verificaInstanciaValida(): void {
        if (empty(self::$instance)) {
            self::iniciaProcesso();
        }
    }
}